/******************************************************************************/
/*
Software(C) Implementation of AES Decryption
Written for 128-bit AES decryption project
Course: CSEE 4840 - Embedded System Design, Spring 2008
Authors: 		Shrivathsa Bhargav (sb2784)
			Larry Chen (lc2454)
			Abhinandan Majumdar (am2993)
			Shiva Ramudit (syr9)

Last modified: 5-8-2008
*/
/******************************************************************************/

#include <stdio.h>

//Used for Debug Prints
//0 -> No Debug Prints
//1 -> With Debug Prints
#define VERBOSE 0

//Variable Declaration for key, text and roundkey
unsigned short int key[4][4];
unsigned short int text[4][4];

unsigned short int roundkey[11][4][4];

//Function to read key file *Should be key.txt*					
void read_key (FILE *f) {
	unsigned short int c=0x00000000;
	void *t = &c;
	int sz;
	int i,j;
	int cn;
	i=j=0;
	
	for (cn=0;cn<16;cn++) {
		sz = fread(t,1,1,f);
		key[i][j++] = c;
		c=0x00000000;
		if (j>=4) {i++;j=0;}
		if(sz==0 || i >= 4) break;
	}
}

//Function to read cipher text file *Supplied as first argument* *Can be recognized by .enc extension*
int read_text (FILE *f) {
	unsigned short int c=0x00000000;
	void *t = &c;
	int sz;
	int i,j;
	int cn;
	
	i=j=0;
	
	for (cn=0;cn<16;cn++) {
		sz = fread(t,1,1,f);
		text[i][j++] = sz?c:0x00;
		c=0x00000000;
	}	
	return feof(f);
}

//Function to display the key
void print_key() {
	int i,j;
	printf("key => \n");
	for(i=0;i<4;i++) {
		for(j=0;j<4;j++)
			printf("%hx ",key[i][j]);
		printf("\n");
	}
}

//Function to display the roundkey
void print_rkey(int count) {
	int i,j;
	printf("key => %d\n",count);
	for(i=0;i<4;i++) {
		for(j=0;j<4;j++)
			printf("%hx ",roundkey[count][i][j]);
		printf("\n");
	}
}

//Function to display the cipher text
void print_text() {
	int i,j;
	printf("text => \n");
	for(i=0;i<4;i++) {
		for(j=0;j<4;j++)
			printf("%hx ",text[i][j]);
		printf("\n");
	}
}

//Declaration for sbox. Used for Key Expansion
unsigned short int sbox[16][16] = {
			0x63, 0x7c, 0x77, 0x7b, 0xf2, 0x6b, 0x6f, 0xc5, 0x30, 0x01, 0x67, 0x2b, 0xfe, 0xd7, 0xab, 0x76,
			0xca, 0x82, 0xc9, 0x7d, 0xfa, 0x59, 0x47, 0xf0, 0xad, 0xd4, 0xa2, 0xaf, 0x9c, 0xa4, 0x72, 0xc0, 
			0xb7, 0xfd, 0x93, 0x26, 0x36, 0x3f, 0xf7, 0xcc, 0x34, 0xa5, 0xe5, 0xf1, 0x71, 0xd8, 0x31, 0x15, 
			0x04, 0xc7, 0x23, 0xc3, 0x18, 0x96, 0x05, 0x9a, 0x07, 0x12, 0x80, 0xe2, 0xeb, 0x27, 0xb2, 0x75, 
			0x09, 0x83, 0x2c, 0x1a, 0x1b, 0x6e, 0x5a, 0xa0, 0x52, 0x3b, 0xd6, 0xb3, 0x29, 0xe3, 0x2f, 0x84, 
			0x53, 0xd1, 0x00, 0xed, 0x20, 0xfc, 0xb1, 0x5b, 0x6a, 0xcb, 0xbe, 0x39, 0x4a, 0x4c, 0x58, 0xcf, 
			0xd0, 0xef, 0xaa, 0xfb, 0x43, 0x4d, 0x33, 0x85, 0x45, 0xf9, 0x02, 0x7f, 0x50, 0x3c, 0x9f, 0xa8, 
			0x51, 0xa3, 0x40, 0x8f, 0x92, 0x9d, 0x38, 0xf5, 0xbc, 0xb6, 0xda, 0x21, 0x10, 0xff, 0xf3, 0xd2, 
			0xcd, 0x0c, 0x13, 0xec, 0x5f, 0x97, 0x44, 0x17, 0xc4, 0xa7, 0x7e, 0x3d, 0x64, 0x5d, 0x19, 0x73, 
			0x60, 0x81, 0x4f, 0xdc, 0x22, 0x2a, 0x90, 0x88, 0x46, 0xee, 0xb8, 0x14, 0xde, 0x5e, 0x0b, 0xdb, 
			0xe0, 0x32, 0x3a, 0x0a, 0x49, 0x06, 0x24, 0x5c, 0xc2, 0xd3, 0xac, 0x62, 0x91, 0x95, 0xe4, 0x79, 
			0xe7, 0xc8, 0x37, 0x6d, 0x8d, 0xd5, 0x4e, 0xa9, 0x6c, 0x56, 0xf4, 0xea, 0x65, 0x7a, 0xae, 0x08, 
			0xba, 0x78, 0x25, 0x2e, 0x1c, 0xa6, 0xb4, 0xc6, 0xe8, 0xdd, 0x74, 0x1f, 0x4b, 0xbd, 0x8b, 0x8a, 
			0x70, 0x3e, 0xb5, 0x66, 0x48, 0x03, 0xf6, 0x0e, 0x61, 0x35, 0x57, 0xb9, 0x86, 0xc1, 0x1d, 0x9e, 
			0xe1, 0xf8, 0x98, 0x11, 0x69, 0xd9, 0x8e, 0x94, 0x9b, 0x1e, 0x87, 0xe9, 0xce, 0x55, 0x28, 0xdf, 
			0x8c, 0xa1, 0x89, 0x0d, 0xbf, 0xe6, 0x42, 0x68, 0x41, 0x99, 0x2d, 0x0f, 0xb0, 0x54, 0xbb, 0x16
			};

//Declaration for inverse sbox. Used for Decryption 						
unsigned short int inv_sbox[16][16] = {
			0x52, 0x09, 0x6a, 0xd5, 0x30, 0x36, 0xa5, 0x38, 0xbf, 0x40, 0xa3, 0x9e, 0x81, 0xf3, 0xd7, 0xfb, 
			0x7c, 0xe3, 0x39, 0x82, 0x9b, 0x2f, 0xff, 0x87, 0x34, 0x8e, 0x43, 0x44, 0xc4, 0xde, 0xe9, 0xcb,
			0x54, 0x7b, 0x94, 0x32, 0xa6, 0xc2, 0x23, 0x3d, 0xee, 0x4c, 0x95, 0x0b, 0x42, 0xfa, 0xc3, 0x4e,
			0x08, 0x2e, 0xa1, 0x66, 0x28, 0xd9, 0x24, 0xb2, 0x76, 0x5b, 0xa2, 0x49, 0x6d, 0x8b, 0xd1, 0x25,
			0x72, 0xf8, 0xf6, 0x64, 0x86, 0x68, 0x98, 0x16, 0xd4, 0xa4, 0x5c, 0xcc, 0x5d, 0x65, 0xb6, 0x92,
			0x6c, 0x70, 0x48, 0x50, 0xfd, 0xed, 0xb9, 0xda, 0x5e, 0x15, 0x46, 0x57, 0xa7, 0x8d, 0x9d, 0x84,
			0x90, 0xd8, 0xab, 0x00, 0x8c, 0xbc, 0xd3, 0x0a, 0xf7, 0xe4, 0x58, 0x05, 0xb8, 0xb3, 0x45, 0x06,
			0xd0, 0x2c, 0x1e, 0x8f, 0xca, 0x3f, 0x0f, 0x02, 0xc1, 0xaf, 0xbd, 0x03, 0x01, 0x13, 0x8a, 0x6b,
			0x3a, 0x91, 0x11, 0x41, 0x4f, 0x67, 0xdc, 0xea, 0x97, 0xf2, 0xcf, 0xce, 0xf0, 0xb4, 0xe6, 0x73,
			0x96, 0xac, 0x74, 0x22, 0xe7, 0xad, 0x35, 0x85, 0xe2, 0xf9, 0x37, 0xe8, 0x1c, 0x75, 0xdf, 0x6e,
			0x47, 0xf1, 0x1a, 0x71, 0x1d, 0x29, 0xc5, 0x89, 0x6f, 0xb7, 0x62, 0x0e, 0xaa, 0x18, 0xbe, 0x1b,
			0xfc, 0x56, 0x3e, 0x4b, 0xc6, 0xd2, 0x79, 0x20, 0x9a, 0xdb, 0xc0, 0xfe, 0x78, 0xcd, 0x5a, 0xf4,
			0x1f, 0xdd, 0xa8, 0x33, 0x88, 0x07, 0xc7, 0x31, 0xb1, 0x12, 0x10, 0x59, 0x27, 0x80, 0xec, 0x5f,
			0x60, 0x51, 0x7f, 0xa9, 0x19, 0xb5, 0x4a, 0x0d, 0x2d, 0xe5, 0x7a, 0x9f, 0x93, 0xc9, 0x9c, 0xef,
			0xa0, 0xe0, 0x3b, 0x4d, 0xae, 0x2a, 0xf5, 0xb0, 0xc8, 0xeb, 0xbb, 0x3c, 0x83, 0x53, 0x99, 0x61,
			0x17, 0x2b, 0x04, 0x7e, 0xba, 0x77, 0xd6, 0x26, 0xe1, 0x69, 0x14, 0x63, 0x55, 0x21, 0x0c, 0x7d 
			};

//Function for inverse_sub_bytes			
void inv_sub_bytes() {
	int i,j,ri,ci;
	
	i=j=ri=ci=0;
	
	for(i=0;i<4;i++)
		for(j=0;j<4;j++) {
			ri = text[i][j] >> 4;
			ci = text[i][j] & 0x0F;
			
			text[i][j] = inv_sbox[ri][ci];
		}
}


//Function for inverse_shift_row
void inv_shift_row() {
	int i,j;
	int t,count;
	
	for (i=1;i<4;i++)
		for(count=0;count<i;count++) {
			t=text[i][3];
			for(j=3;j>0;j--) 
				text[i][j]=text[i][j-1];
			text[i][j]=t;
		}
}


//Function for inverse_mix_column

// xtime is a macro that finds the product of {02} and the argument to xtime modulo {1b} 
#define xtime(x)   ((x<<1) ^ (((x>>7) & 1) * 0x1b))
// Multiplty is a macro used to multiply numbers in the field GF(2^8)
#define Multiply(x,y) (((y & 1) * x) ^ ((y>>1 & 1) * xtime(x)) ^ ((y>>2 & 1) * xtime(xtime(x))) ^ ((y>>3 & 1) * xtime(xtime(xtime(x)))) ^ ((y>>4 & 1) * xtime(xtime(xtime(xtime(x))))))

void inv_mix_column() {
	int i;
	unsigned short int a,b,c,d;
	for(i=0;i<4;i++) { 
		a = text[0][i];
		b = text[1][i];
		c = text[2][i];
		d = text[3][i];
			
		text[0][i] = 0xFF & (Multiply(a, 0x0e) ^ Multiply(b, 0x0b) ^ Multiply(c, 0x0d) ^ Multiply(d, 0x09));
		text[1][i] = 0xFF & (Multiply(a, 0x09) ^ Multiply(b, 0x0e) ^ Multiply(c, 0x0b) ^ Multiply(d, 0x0d));
		text[2][i] = 0xFF & (Multiply(a, 0x0d) ^ Multiply(b, 0x09) ^ Multiply(c, 0x0e) ^ Multiply(d, 0x0b));
		text[3][i] = 0xFF & (Multiply(a, 0x0b) ^ Multiply(b, 0x0d) ^ Multiply(c, 0x09) ^ Multiply(d, 0x0e));
	}
}


//Function for inverse_add_roundkey
void inv_add_roundkey(int count) {
	int i,j;
	
	for(i=0;i<4;i++)
		for(j=0;j<4;j++)
			text[i][j]^=roundkey[count][i][j];
}


//Function for key_schedule (for one iteration)
void key_schedule(int count) {
	unsigned short int Rcon[10] = {0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80,0x1b,0x36};
	int i,j,ri,ci;
	unsigned short int t,a[4];
	
	for (j=3,i=0;i<4;i++)
		a[i] = roundkey[count-1][i][j];
	
	/* rotate column */
	t=a[0];
	for (i=0;i<3;i++)
		a[i]=a[i+1];
	
	a[i]=t;
	
	/* sub_bytes */
	for(i=0;i<4;i++) {
		ri = a[i] >> 4;
		ci = a[i] & 0x0F;
		a[i] = sbox[ri][ci];
	}
	
	/*1st xor */
	for(j=0,i=0;i<4;i++) {
		if(i==0)
			roundkey[count][i][j] = Rcon[count-1] ^ roundkey[count-1][i][j] ^ a[i];
		else
			roundkey[count][i][j] = roundkey[count-1][i][j] ^ a[i];
	}
	
	/*last xor */
	for(j=1;j<4;j++)
		for(i=0;i<4;i++)
			roundkey[count][i][j]=roundkey[count][i][j-1] ^ roundkey[count-1][i][j];
}

//Function for key_expansion
void keyexpand() {
	int i,j;
	int count = 0;
	
	for(i=0;i<4;i++)
		for(j=0;j<4;j++)
			roundkey[count][i][j]=key[i][j];
			for(count=1;count<11;count++) {
				key_schedule(count);
			}
}

//Function for Decryption of 128bit block
void decrypt_block() {
	int count = 10;
	keyexpand();
	inv_add_roundkey(count);
	
	#if VERBOSE
	printf("\n*******************Round %d**************************\n",count);
	print_rkey(count);
	print_text();
	#endif
	
	for (count = 9; count > 0 ; count--) {
		inv_shift_row();
		inv_sub_bytes();
		inv_add_roundkey(count);
		inv_mix_column();
		
		#if VERBOSE
		printf("\n*******************Round %d**************************\n",count);
		print_rkey(count);
		print_text();
		#endif
	}
	
	inv_shift_row();
	inv_sub_bytes();
	inv_add_roundkey(count);
	
	#if VERBOSE
	
	printf("\n*******************Round %d**************************\n",count);
	print_rkey(count);
	print_text();
	#endif
}

//Function to write the generated plain text (decrypted data) into a file (with extension .text)
void write_cipher(FILE *f) {
	int i,j;
	void *t;
	
	for(i=0;i<4;i++)
		for(j=0;j<4;j++) {
			t = &text[i][j];
			fwrite(t,1,1,f);
		}
}

//Main Function
int main(int argc, char *argv[1]) {

	FILE *fk = fopen("key.txt","r");
	FILE *ft = fopen(argv[1],"r");
	char destname[strlen(argv[1])+5];
	sprintf(destname,"%s.%s",argv[1],"text");
	FILE *fw = fopen(destname,"w");
	
	int sz=1, count=0;
	
	int filesize,fcount = 0;
	
	fseek(ft,0L,SEEK_END);
	filesize = ftell(ft);
	rewind(ft);
	
	
	#if VERBOSE
	printf("filesize = %d\n",filesize);
	printf("Step1 ...%d\n",feof(ft));
	#endif
	
	
	
	while(fcount < filesize) {
		read_key(fk);
		sz = read_text(ft);
		
		#if VERBOSE
		print_key();
		print_text();
		#endif
		
		decrypt_block();
		
		write_cipher(fw);
		
		rewind(fk);
		count++;
		fcount+=16;
	}
		
	fclose(ft);
	fclose(fk);
	fclose(fw);
}




