-------------------------------------------------------------------------------
--
-- Expansion Keys Module
-- Written for 128-bit AES decryption project
-- Course: CSEE 4840 - Embedded System Design, Spring 2008
-- Authors: Shrivathsa Bhargav (sb2784)
--			Larry Chen (lc2454)
--			Abhinandan Majumdar (am2993)
--			Shiva Ramudit (syr9)
--
-- Last modified: 5-8-2008
-------------------------------------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

--Central Module for Key Expansion -- 
--All round keys are written here during key generation and read during decryption
entity expansion_keys is
	port (
		clk, en : in std_logic;
		writeaddr : in std_logic_vector (3 downto 0);
		addr : in std_logic_vector (3 downto 0);
		it : in std_logic_vector (127 downto 0);
		key : out std_logic_vector (127 downto 0);
		writeComplete : out std_logic
		);
end;

architecture expansion_keys_arch of expansion_keys is
type key_table_row is array(0 to 3) of std_logic_vector(7 downto 0);
type key_table_type is array(0 to 3) of key_table_row;
type key_table_array is array(0 to 11) of key_table_type;

signal key_table : key_table_array; 

signal	in00, in01, in02, in03 : std_logic_vector (7 downto 0);
signal	in10, in11, in12, in13 : std_logic_vector (7 downto 0);
signal	in20, in21, in22, in23 : std_logic_vector (7 downto 0);
signal	in30, in31, in32, in33 : std_logic_vector (7 downto 0);
signal  key00, key01, key02, key03 : std_logic_vector (7 downto 0);
signal  key10, key11, key12, key13 : std_logic_vector (7 downto 0);
signal  key20, key21, key22, key23 : std_logic_vector (7 downto 0);
signal  key30, key31, key32, key33 : std_logic_vector (7 downto 0);

begin

process (clk) 

begin

if rising_edge(clk) then

	if en = '1' then
		
		-- Writing the intermediate key for a specific iteration into 
		-- the corresponding address of key-table
		key_table(conv_integer(writeaddr))(0)(0) <= in00;
		key_table(conv_integer(writeaddr))(0)(1) <= in01;
		key_table(conv_integer(writeaddr))(0)(2) <= in02;
		key_table(conv_integer(writeaddr))(0)(3) <= in03;
		key_table(conv_integer(writeaddr))(1)(0) <= in10;
		key_table(conv_integer(writeaddr))(1)(1) <= in11;
		key_table(conv_integer(writeaddr))(1)(2) <= in12;
		key_table(conv_integer(writeaddr))(1)(3) <= in13;
		key_table(conv_integer(writeaddr))(2)(0) <= in20;
		key_table(conv_integer(writeaddr))(2)(1) <= in21;
		key_table(conv_integer(writeaddr))(2)(2) <= in22;
		key_table(conv_integer(writeaddr))(2)(3) <= in23;
		key_table(conv_integer(writeaddr))(3)(0) <= in30;
		key_table(conv_integer(writeaddr))(3)(1) <= in31;
		key_table(conv_integer(writeaddr))(3)(2) <= in32;
		key_table(conv_integer(writeaddr))(3)(3) <= in33;
		
		if writeaddr = "0000" then
			writecomplete <= '1';
		else
			writecomplete <= '0';
		end if;
		
	end if;

end if;

end process;

--Splitting 128 bit bus into individual 8 bit lines
in00 <= it(127 downto 120);
in01 <= it(119 downto 112);
in02 <= it(111 downto 104);
in03 <= it(103 downto 96);
in10 <= it(95 downto 88);
in11 <= it(87 downto 80);
in12 <= it(79 downto 72);
in13 <= it(71 downto 64);
in20 <= it(63 downto 56);
in21 <= it(55 downto 48);
in22 <= it(47 downto 40);
in23 <= it(39 downto 32);
in30 <= it(31 downto 24);
in31 <= it(23 downto 16);
in32 <= it(15 downto 8);
in33 <= it(7 downto 0);
					
--Fetching the intermediate key-data from the key-table
key00 <= key_table(conv_integer(addr))(0)(0);
key01 <= key_table(conv_integer(addr))(0)(1);
key02 <= key_table(conv_integer(addr))(0)(2);
key03 <= key_table(conv_integer(addr))(0)(3);
key10 <= key_table(conv_integer(addr))(1)(0);
key11 <= key_table(conv_integer(addr))(1)(1);
key12 <= key_table(conv_integer(addr))(1)(2);
key13 <= key_table(conv_integer(addr))(1)(3);
key20 <= key_table(conv_integer(addr))(2)(0);
key21 <= key_table(conv_integer(addr))(2)(1);
key22 <= key_table(conv_integer(addr))(2)(2);
key23 <= key_table(conv_integer(addr))(2)(3);
key30 <= key_table(conv_integer(addr))(3)(0);
key31 <= key_table(conv_integer(addr))(3)(1);
key32 <= key_table(conv_integer(addr))(3)(2);
key33 <= key_table(conv_integer(addr))(3)(3);

--Putting individual 8 bits into 128 bit bus
key <= key00 & key01 & key02 & key03 &
		 key10 & key11 & key12 & key13 &
		 key20 & key21 & key22 & key23 &
		 key30 & key31 & key32 & key33 ;
			
end expansion_keys_arch;
					
				