-------------------------------------------------------------------------------
--
-- Roundkey Generator Module
-- Written for 128-bit AES decryption project
-- Course: CSEE 4840 - Embedded System Design, Spring 2008
-- Authors: Shrivathsa Bhargav (sb2784)
--			Larry Chen (lc2454)
--			Abhinandan Majumdar (am2993)
--			Shiva Ramudit (syr9)
--
-- Last modified: 5-8-2008
-------------------------------------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Generates each rounds key
entity generate_roundkey is
	port (
		count : in std_logic_vector(3 downto 0);
		it : in std_logic_vector(127 downto 0);
		roundkey : out std_logic_vector(127 downto 0)
		);
end;

architecture roundkey_arch of generate_roundkey is

-- Used to store the rcon data
type rcon_row_type is array (0 to 3) of std_logic_vector (7 downto 0);
type rcon_type is array (0 to 9) of rcon_row_type;

-- Rcon is hard coded into a Rom
constant rcon_box : rcon_type := 
		(
			(x"01", x"00", x"00", x"00"),
			(x"02", x"00", x"00", x"00"),
			(x"04", x"00", x"00", x"00"),
			(x"08", x"00", x"00", x"00"),
			(x"10", x"00", x"00", x"00"),
			(x"20", x"00", x"00", x"00"),
			(x"40", x"00", x"00", x"00"),
			(x"80", x"00", x"00", x"00"),
			(x"1b", x"00", x"00", x"00"),
			(x"36", x"00", x"00", x"00")
		);

-- Stores the rotated last column
signal rotword0, rotword1, rotword2, rotword3 : std_logic_vector(7 downto 0);

-- Stores the subbytes result of the previous
signal subres0, subres1, subres2, subres3 : std_logic_vector(7 downto 0);

signal 	temp00, temp01, temp02, temp03,
		temp10, temp11, temp12, temp13,
		temp20, temp21, temp22, temp23,
		temp30, temp31, temp32, temp33: std_logic_vector (7 downto 0);

-- Breaks up input key
signal it00, it01, it02, it03 : std_logic_vector (7 downto 0);
signal it10, it11, it12, it13 : std_logic_vector (7 downto 0);
signal it20, it21, it22, it23 : std_logic_vector (7 downto 0);
signal it30, it31, it32, it33 : std_logic_vector (7 downto 0);


begin

it00 <= it(127 downto 120);
it01 <= it(119 downto 112);
it02 <= it(111 downto 104);
it03 <= it(103 downto 96);
it10 <= it(95 downto 88);
it11 <= it(87 downto 80);
it12 <= it(79 downto 72);
it13 <= it(71 downto 64);
it20 <= it(63 downto 56);
it21 <= it(55 downto 48);
it22 <= it(47 downto 40);
it23 <= it(39 downto 32);
it30 <= it(31 downto 24);
it31 <= it(23 downto 16);
it32 <= it(15 downto 8);
it33 <= it(7 downto 0);

rotword0 <= it13;
rotword1 <= it23;
rotword2 <= it33;
rotword3 <= it03;

-- Performs Sbox transform
SUBBOX : entity work.sbox port map (
		  it0 => rotword0, it1 => rotword1, it2 => rotword2, it3 => rotword3, 
		  ot0 => subres0, ot1 => subres1, ot2 => subres2, ot3 => subres3
);


temp00 <= it00 xor subres0 xor rcon_box(conv_integer(count))(0);
temp10 <= it10 xor subres1 xor rcon_box(conv_integer(count))(1);
temp20 <= it20 xor subres2 xor rcon_box(conv_integer(count))(2);
temp30 <= it30 xor subres3 xor rcon_box(conv_integer(count))(3);

temp01 <= it01 xor temp00;
temp11 <= it11 xor temp10;
temp21 <= it21 xor temp20;
temp31 <= it31 xor temp30;

temp02 <= it02 xor temp01;
temp12 <= it12 xor temp11;
temp22 <= it22 xor temp21;
temp32 <= it32 xor temp31;

temp03 <= it03 xor temp02;
temp13 <= it13 xor temp12;
temp23 <= it23 xor temp22;
temp33 <= it33 xor temp32;

roundkey <= temp00 & temp01 & temp02 & temp03 &
		 temp10 & temp11 & temp12 & temp13 &
		 temp20 & temp21 & temp22 & temp23 &
		 temp30 & temp31 & temp32 & temp33 ;


end roundkey_arch;