-------------------------------------------------------------------------------
--
-- Inverse Multiply Module (Part of Inverse Mix Columns Module)
-- Written for 128-bit AES decryption project
-- Course: CSEE 4840 - Embedded System Design, Spring 2008
-- Authors: Shrivathsa Bhargav (sb2784)
--			Larry Chen (lc2454)
--			Abhinandan Majumdar (am2993)
--			Shiva Ramudit (syr9)
--
-- Last modified: 5-8-2008
-------------------------------------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Module to perform mtimes calculation - modularized to reuse parts within calculation
entity inv_multiply is
	port (
		input : in std_logic_vector (7 downto 0);
		const : in std_logic_vector (7 downto 0);
		output : out std_logic_vector (7 downto 0)
		);
end;

architecture inv_multiply_arch of inv_multiply is

-- Store Mtimes results
signal mtimes1 : std_logic_vector (7 downto 0);
signal mtimes2 : std_logic_vector (7 downto 0);
signal mtimes3 : std_logic_vector (7 downto 0);

-- Store const >> # * mtimes^#(input)
signal term0 : std_logic_vector (7 downto 0);
signal term1 : std_logic_vector (7 downto 0);
signal term2 : std_logic_vector (7 downto 0);
signal term3 : std_logic_vector (7 downto 0);

begin

-- mtimes(input)
Calc1: entity work.inv_mtimes port map(
		input => input, output => mtimes1
);
-- mtimes(mtimes(input))
Calc2: entity work.inv_mtimes port map(
		input => mtimes1, output => mtimes2
);
-- mtimes(mtimes(mtimes(input)))
Calc3: entity work.inv_mtimes port map(
		input => mtimes2 , output => mtimes3
);

-- (const & 1) * input
term0 <= input when const(0) = '1' else
     (others => '0');
-- (const >> 1 & 1) * mtimes(input)
term1 <= mtimes1 when const(1) = '1' else
     (others => '0');
-- (const >> 2 & 1) * mtimes(mtimes(input))
term2 <= mtimes2 when const(2) = '1' else
     (others => '0');
-- (const >> 3 & 1) * mtimes(mtimes(mtimes(input)))
term3 <= mtimes3 when const(3) = '1' else
     (others => '0');

output <= term0 xor term1 xor term2 xor term3;

end inv_multiply_arch; 