-------------------------------------------------------------------------------
--
-- Key Generation Controller
-- Written for 128-bit AES decryption project
-- Course: CSEE 4840 - Embedded System Design, Spring 2008
-- Authors: Shrivathsa Bhargav (sb2784)
--			Larry Chen (lc2454)
--			Abhinandan Majumdar (am2993)
--			Shiva Ramudit (syr9)
--
-- Last modified: 5-8-2008
-------------------------------------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

--Module for Key Controller
entity key_controller is
port (
	clk, en : in std_logic;
	count   : out std_logic_vector(3 downto 0);
	e0,e1,e2,eoc   : out std_logic
	);
end;

architecture key_controller_arch of key_controller is

type state_type is (S0, S1, S2, S3, S4);
signal state : state_type := S0;

signal t_count : std_logic_vector(3 downto 0) := "0000";
signal twocycle : std_logic_vector(1 downto 0) := "00";

begin

process (clk) 
begin

if rising_edge(clk) then

	case state is

		-- First State - Store key in the SRAM
		when S0 =>
					if en = '1' then
						
						t_count <= t_count + '1';
						state <= S1;
						
					end if;


		-- Second State - Select Key for calculation of round key
		when S1 => 
					if en = '1' then
						
						t_count <= t_count + '1';
						state <= S2;
						
					end if;
		
		-- Third State - Select Last Round Key for calculation of Current Round Key
		when S2 => 
					
					if en = '1' then
						if t_count = "1001" then
							state <= S3;
						else
							t_count <= t_count + '1';
						end if;
					end if;
					
		-- Added to wait two cycles for writing
		when S3 => 
					if en = '1' then
						if twocycle = "01" then
							twocycle <= twocycle;
							state <= S4;
						else
							twocycle <= twocycle + '1';
						end if;
					end if;
					
		when S4 =>
					if en = '1' then
						state <= S4;
					elsif en = '0' then
						state <= S0;
						t_count <= "0000";
						twocycle <= "00";					
					end if;
		            
	end case;
   

end if;

end process;

-- Iteration Count
count <= t_count; 

--Setting the control bits required for Key-Controller Datapath
e0 <= '0' when state = S0 and en = '1' else
	  '1' when state = S1 and en = '1' else
	  '1' when state = S2 and en = '1' else
	  '1' when state = S3 and en = '1' else
	  '0';
	
e1 <= '1' when state = S0 and en = '1' else
      '1' when state = S1 and en = '1' else
	  '1' when state = S2 and en = '1' else
	  '0' when state = S3 and en = '1' else
	  '0';

e2 <= '0' when state = S0 and en = '1' else
      '1' when state = S1 and en = '1' else
	  '1' when state = S2 and en = '1' else
	  '1' when state = S3 and en = '1' else
	  '1';

eoc <= '1' when state = S0 and en = '1' else
       '1' when state = S1 and en = '1' else
	   '1' when state = S2 and en = '1' else
	   '1' when state = S3 and en = '1' else
	   '0';
end key_controller_arch;